from .consulta import (
    CDA,
    OAB,
    CheckResults,
    ConsultaPor,
    ConsultaProcesso,
    Consultar,
    DocumentoParte,
    NomeAdvogado,
    NomeParte,
    NumCartaPrecatoria,
    NumDocDelegacia,
    NumeroUnificado,
)
from .registro import Header, Partes, Movimentacao

# import .registro



"""
Módulo com a página do registro de Processos,
ou seja, dados de Partes, Movimentação etc
"""

from selenium.webdriver.common.by import By

from pyesaj.scraper.pom import PageElement


# from selenium.webdriver.common.keys import Keys
# from selenium.webdriver.common.action_chains import ActionChains


# flake8: noqa:303
# flake8: noqa:E501


class Header(PageElement):
    """
    Representa o Header, ou seja, as informações do processo.
    Deverá servir para processos de Primeiro e Segundo Graus
    """

    # Informação Básica
    classe = (By.XPATH, '//*[@id="classeProcesso"]')
    assunto = (By.XPATH, '//*[@id="assuntoProcesso"]')
    foro = (By.XPATH, '//*[@id="foroProcesso"]')
    vara = (By.XPATH, '//*[@id="varaProcesso"]')
    juiz = (By.XPATH, '//*[@id="juizProcesso"]')

    # Após Clicar em "Ver Mais"
    distribuicao = (By.XPATH, '//*[@id="dataHoraDistribuicaoProcesso"]')
    n_controle = (By.XPATH, '//*[@id="numeroControleProcesso"]')
    area = (By.XPATH, '//*[@id="areaProcesso"]')
    valor_acao = (By.XPATH, '//*[@id="valorAcaoProcesso"]')

    # Outros
    outros_numeros = (
        By.XPATH,
        '//*[@class="unj-label" and contains(text(),"Outros números")]/following-sibling::div',
    )
    local_fisico = (
        By.XPATH,
        '//*[@class="unj-label" and contains(text(),"Local Físico")]/following-sibling::div',
    )
    outros_assuntos = (
        By.XPATH,
        '//*[@class="unj-label" and contains(text(),"Outros assuntos")]/following-sibling::div',
    )

    # Ver Mais
    ver_mais = (
        By.XPATH,
        '//div[@class="unj-entity-header"]//*[@href="#maisDetalhes"]',
    )

    def get_class(self) -> None:
        """
        Define o "Nº do Documento da Delegacia".
        """
        self.clica_ver_mais()
        dd_dados = {
            'classe': self.get_text(locator=self.classe),
            'assunto': self.get_text(locator=self.assunto),
            'foro': self.get_text(locator=self.foro),
            'vara': self.get_text(locator=self.vara),
            'juiz': self.get_text(locator=self.juiz),
            # Ver Mais
            'distribuicao': self.get_text(locator=self.distribuicao, wait=1),
            'n_controle': self.get_text(locator=self.n_controle, wait=1),
            'area': self.get_text(locator=self.area, wait=1),
            'valor_acao': self.get_text(locator=self.valor_acao, wait=1),
            # Outros
            'outros_numeros': self.get_text(
                locator=self.outros_numeros, wait=1
            ),
            'local_fisico': self.get_text(locator=self.local_fisico, wait=1),
            'outros_assuntos': self.get_text(
                locator=self.outros_assuntos, wait=1
            ),
        }

        # Ver Menos / Recolher
        self.clica_ver_menos()
        return dd_dados

    def clica_ver_mais(self):
        if (
            self.attribute(locator=self.ver_mais, attribute='aria-expanded')
            == 'false'
        ):
            self.click(locator=self.ver_mais)

    def clica_ver_menos(self):
        if (
            self.attribute(locator=self.ver_mais, attribute='aria-expanded')
            != 'false'
        ):
            self.click(locator=self.ver_mais)


class Partes(PageElement):
    """
    Representa a Sessão que contem as Partes do Processo.
    Deverá servir para processos de Primeiro e Segundo Graus
    Por ora só funciona para processos de 1º grau!
    """

    # Tabela
    tabela_linhas = (By.XPATH, '//table[@id="tableTodasPartes"]//tbody//tr')

    # Ver Mais
    ver_mais_loc = (
        By.XPATH,
        '//*[@id="divLinksTituloBlocoPartes"]',
    )
    ver_mais = (
        By.XPATH,
        '//*[@id="linkpartes"]//i',
    )

    def get_partes(self) -> None:
        """
        Define o "Nº do Documento da Delegacia".
        """

        self.clica_ver_mais()

        rows = self.find_all(locator=self.tabela_linhas)
        dd = []
        for i in rows:
            list_itens = i.text.strip().split('\n', maxsplit=1)
            for item in list_itens:
                # print(item)

                if item.startswith('Reqte  '):
                    item = item.replace('Reqte  ', '').strip()
                    # print(item)
                    dd.append({'requerente': item})

                if item.startswith('Reqdo  '):
                    item = item.replace('Reqdo  ', '').strip()
                    # print(item)
                    dd.append({'requerido': item})

                if item.startswith('Advogada:  ') or item.startswith(
                    'Advogado:  '
                ):
                    item = (
                        item.replace('Advogada:  ', '')
                        .replace('Advogado:  ', '')
                        .strip()
                    )
                    # print(item)
                    dd.append({'advogado': item})

        self.clica_ver_menos()
        return dd

    def clica_ver_mais(self):
        # aaa = self.find(locator=self.ver_mais)
        # self.script("arguments[0].scrollIntoView();", aaa)
        self.script("scrollBy(0,0);")

        # Vai até o botão do Ver Mais
        self.script(
            "arguments[0].scrollIntoView();",
            self.find(locator=self.ver_mais_loc),
        )
        # Sobe um pouco
        self.driver.execute_script("window.scrollBy(0, -250);")

        if (
            self.attribute(locator=self.ver_mais, attribute='class')
            == 'unj-link-collapse__icon glyph glyph-chevron-down'
        ):
            self.click(locator=self.ver_mais)

            # Vai até o botão do Ver Mais
            self.script(
                "arguments[0].scrollIntoView();",
                self.find(locator=self.ver_mais_loc),
            )
            # Sobe um pouco
            self.driver.execute_script("window.scrollBy(0, -250);")

    def clica_ver_menos(self):
        # Vai até o botão do Ver Mais
        self.script(
            "arguments[0].scrollIntoView();",
            self.find(locator=self.ver_mais_loc),
        )
        # Sobe um pouco
        self.driver.execute_script("window.scrollBy(0, -250);")
        # self.script("scrollBy(0,0);")

        if (
            self.attribute(locator=self.ver_mais, attribute='class')
            != 'unj-link-collapse__icon glyph glyph-chevron-down'
        ):
            self.click(locator=self.ver_mais)

            # Vai até o botão do Ver Mais
            self.script(
                "arguments[0].scrollIntoView();",
                self.find(locator=self.ver_mais_loc),
            )
            # Sobe um pouco
            self.driver.execute_script("window.scrollBy(0, -250);")




class Movimentacao(PageElement):
    """
    Representa a Sessão que contem as Partes do Processo.
    Deverá servir para processos de Primeiro e Segundo Graus
    Por ora só funciona para processos de 1º grau!
    """

    # Tabela
    tabela_linhas = (By.XPATH, '//tbody[@id="tabelaTodasMovimentacoes"]//tr')
    row_data_mov = (By.XPATH, './/td[@class="dataMovimentacao"]')
    row_desc_mov = (By.XPATH, './/td[@class="descricaoMovimentacao"]')

    # Ver Mais
    ver_mais_loc = (
        By.XPATH,
        '//*[@id="divLinksTituloBlocoMovimentacoes"]',
    )

    ver_mais = (
        By.XPATH,
        '//*[@id="linkmovimentacoes"]//i',
    )

    def get_table(self) -> None:
        """
        Define o "Nº do Documento da Delegacia".
        """

        self.clica_ver_mais()

        rows = self.find_all(locator=self.tabela_linhas)
        dd = []
        for row in rows:
            # print(row.text)
            data_xpath = self.row_data_mov + (row,)
            data = self.get_text(locator=data_xpath, wait=3)
            # print(data)

            desc_xpath = self.row_desc_mov + (row,)
            desc = self.get_text(locator=desc_xpath, wait=3)
            # print(desc)

            dd.append({'data_movimentacao': data, 'descricao': desc})

        self.clica_ver_menos()
        return dd

    def clica_ver_mais(self):
        # Vai até o botão do Ver Mais
        self.script(
            "arguments[0].scrollIntoView();",
            self.find(locator=self.ver_mais_loc),
        )
        # Sobe um pouco
        self.driver.execute_script("window.scrollBy(0, -250);")

        if (
            self.attribute(locator=self.ver_mais, attribute='class')
            == 'unj-link-collapse__icon glyph glyph-chevron-down'
        ):

            # Clica no Botão Ver Mais
            self.click(locator=self.ver_mais)

            # Vai até o botão do Ver Mais
            self.script(
                "arguments[0].scrollIntoView();",
                self.find(locator=self.ver_mais_loc),
            )
            # Sobe um pouco
            self.driver.execute_script("window.scrollBy(0, -250);")

    def clica_ver_menos(self):
        # Vai até o botão do Ver Mais
        self.script(
            "arguments[0].scrollIntoView();",
            self.find(locator=self.ver_mais_loc),
        )
        # Sobe um pouco
        self.driver.execute_script("window.scrollBy(0, -250);")

        if (
            self.attribute(locator=self.ver_mais, attribute='class')
            != 'unj-link-collapse__icon glyph glyph-chevron-down'
        ):
            self.click(locator=self.ver_mais)

            # Vai até o botão do Ver Mais
            self.script(
                "arguments[0].scrollIntoView();",
                self.find(locator=self.ver_mais_loc),
            )
            # Sobe um pouco
            self.driver.execute_script("window.scrollBy(0, -250);")




if __name__ == '__main__':
    pass



"""
Módulo para definição de objetos
que serão os parâmetros de input
das pesquisas na seção de Intimações

TODO: Avaliar o uso
https://stackoverflow.com/questions/76537360/initialize-one-of-two-pydantic-models-depending-on-an-init-parameter
"""

from datetime import date
from typing import Literal, Optional

from pydantic import BaseModel, ValidationInfo, field_validator, model_validator


class RecebeIntimacoes(BaseModel):
    """
    Parâmetros de pesquisa de Intimações das instâncias do tipo:
    - 1º Grau
    - 2º Grau
    - Turmas Recursais
    """

    # TODO: Fazer com que seja possível definir a classe programaticamente.
    em_nome_de: Literal['Ministério Público do Estado de São Paulo', 'Outros']
    tipo_participacao: Optional[str] = None
    instancia: Literal['Primeiro Grau', 'Segundo Grau', 'Turmas Recursais']

    # TODO: Não pode marcar Vara se não marcar Foro
    foro: Optional[str] = None  # Exclusivo 1° Grau
    vara: Optional[str] = None  # Exclusivo 1° Grau

    # TODO: Não pode marcar Órgão Julgador se não marcar Seção
    secao: Optional[str] = None  # Exclusivo 2° Grau
    orgao_julgador: Optional[str] = None  # Exclusivo 2° Grau

    especializacao: Optional[str] = None
    especializacao_nao_definida: Optional[bool] = False
    cargo: Optional[str] = None
    cargo_nao_definido: Optional[bool] = False
    classe: Optional[str] = None  # Exclusivo 2° Grau
    assunto_principal: Optional[str] = None  # Exclusivo 2° Grau
    area: Literal['Ambas', 'Cível', 'Criminal']
    processo: Optional[str] = None
    natureza_comunicacao: Literal['Intimação', 'Citação', 'Ambas']

    @model_validator(mode='after')
    def check_especializacao(self):
        """
        Precisa ter "Especialização" ou
        "Especialização Não Definida".

        Não pode ter os dois!

        https://github.com/pydantic/pydantic/issues/506
        """
        if self.especializacao and self.especializacao_nao_definida:
            raise ValueError(
                'Foram definidas as variáveis "Especialização" e "Especialização não Definida" como "True".\n'
                'Só pode ser definida uma dessas duas variáveis ou definir como "False".'
            )
        return self

    @model_validator(mode='after')
    def check_cargo(self):
        """
        Precisa ter "Especialização" ou
        "Especialização Não Definida".

        Não pode ter os dois!

        https://github.com/pydantic/pydantic/issues/506
        """
        if self.cargo and self.cargo_nao_definido:
            raise ValueError(
                'Foram definidas as variáveis "Cargo" e "Cargo não Definido" como "True".\n'
                'Só pode ser definida uma dessas duas variáveis ou definir como "False".'
            )
        return self

    @field_validator('foro', 'vara', check_fields=False)
    def set_none(cls, v, info: ValidationInfo):
        """
        Define "None" caso seja de 2º grau
        """
        if info.data['instancia'] in ('Segundo Grau', 'Turmas Recursais'):
            return None
        return v

    @field_validator(
        'secao',
        'orgao_julgador',
        'classe',
        'assunto',
        'area',
        check_fields=False,
    )
    def set_none_2(cls, v, info: ValidationInfo):
        """
        Define "None" caso seja de 1° grau
        """
        if info.data['instancia'] == 'Primeiro Grau':
            return None
        return v


class ConsultaIntimacoes(BaseModel):
    """
    Parâmetros de pesquisa de Intimações das instâncias do tipo:
    - 1º Grau
    - 2º Grau
    - Turmas Recursais
    """

    # TODO: Fazer com que seja possível definir a classe programaticamente.
    em_nome_de: Literal['Ministério Público do Estado de São Paulo', 'Outros']
    tipo_participacao: Optional[str] = None
    instancia: Literal['Primeiro Grau', 'Segundo Grau', 'Turmas Recursais']

    # TODO: Não pode marcar Vara se não marcar Foro
    foro: Optional[str] = None  # Exclusivo 1° Grau
    vara: Optional[str] = None  # Exclusivo 1° Grau

    # TODO: Não pode marcar Órgão Julgador se não marcar Seção
    secao: Optional[str] = None  # Exclusivo 2° Grau
    orgao_julgador: Optional[str] = None  # Exclusivo 2° Grau

    especializacao: Optional[str] = None
    especializacao_nao_definida: Optional[bool] = False
    cargo: Optional[str] = None
    cargo_nao_definido: Optional[bool] = False
    classe: Optional[str] = None  # Exclusivo 2° Grau
    assunto_principal: Optional[str] = None  # Exclusivo 2° Grau
    area: Literal['Ambas', 'Cível', 'Criminal']
    periodo_de: Optional[date] = None
    periodo_ate: Optional[date] = None
    processo: Optional[str] = None
    ciencia_ato: Literal['Todos', 'Automática', 'Portal']
    natureza_comunicacao: Literal['Intimação', 'Citação', 'Ambas']
    situacao: Literal['Cumprida', 'Pendente', 'Ambas']

    @model_validator(mode='after')
    def check_especializacao(self):
        """
        Precisa ter "Especialização" ou
        "Especialização Não Definida".

        Não pode ter os dois!

        https://github.com/pydantic/pydantic/issues/506
        """
        if self.especializacao and self.especializacao_nao_definida:
            raise ValueError(
                'Foram definidas as variáveis "Especialização" e "Especialização não Definida" como "True".\n'
                'Só pode ser definida uma dessas duas variáveis ou definir como "False".'
            )
        return self

    @model_validator(mode='after')
    def check_cargo(self):
        """
        Precisa ter "Cargo" ou
        "Cargo Não Definido".

        Não pode ter os dois!

        https://github.com/pydantic/pydantic/issues/506
        """
        if self.cargo and self.cargo_nao_definido:
            raise ValueError(
                'Foram definidas as variáveis "Cargo" e "Cargo não Definido" como "True".\n'
                'Só pode ser definida uma dessas duas variáveis ou definir como "False".'
            )
        return self

    @field_validator('foro', 'vara', check_fields=False)
    def set_none(cls, v, info: ValidationInfo):
        """
        Define "None" caso seja de 2º grau
        """
        if info.data['instancia'] in ('Segundo Grau', 'Turmas Recursais'):
            return None
        return v

    @field_validator(
        'secao',
        'orgao_julgador',
        'classe',
        'assunto',
        'area',
        check_fields=False,
    )
    def set_none_2(cls, v, info: ValidationInfo):
        """
        Define "None" caso seja de 1° grau
        """
        if info.data['instancia'] == 'Primeiro Grau':
            return None
        return v


if __name__ == '__main__':
    #
    intim_search = RecebeIntimacoes(
        em_nome_de='Ministério Público do Estado de São Paulo',
        # foro='sss',
        # vara='232323213',
        #instancia='Primeiro Grau',
        instancia='Segundo Grau',
        # especializacao='Criminal',
        especializacao_nao_definida=True,
        cargo_nao_definido=True,
        area='Ambas',
        natureza_comunicacao='Ambas',
    )

    # intim_search = ConsultaIntimacoes(
    #     em_nome_de='Ministério Público do Estado de São Paulo',
    #     # foro='sss',
    #     # vara='232323213',
    #     instancia='Primeiro Grau',
    #     # especializacao='Criminal',
    #     especializacao_nao_definida=True,
    #     cargo_nao_definido=True,
    #     area='Ambas',
    #     natureza_comunicacao='Ambas',
    #     ciencia_ato='Todos',
    #     situacao='Ambas',
    # )

    print(intim_search)
    # print(intim_search.em_nome_de)
    print(intim_search.foro)
    print(intim_search.vara)
    print(intim_search.instancia)
    print(intim_search.area)



"""
Módulo para definição de objetos
que serão os parâmetros de output
das pesquisas na seção de Intimações

Trata-se da representação dos registros
da tabela que aparece no e-SAJ
"""

from datetime import date, datetime
from typing import Optional

from pydantic import BaseModel, field_validator


# TODO: Mapear as tarjas
class RecebimentoIntimacoes1Grau(BaseModel):
    """
    Objeto "Intimações" de 1º Grau.

    Muito similar às Intimações de 2º Grau,
    sendo que a única diferença é que existe
    "foro" ao invés de "seção".
    """

    disponibilizacao: Optional[str] = None
    prazo_processual: Optional[int] = None
    numero_processo: Optional[str] = None
    classe_assunto_principal: Optional[str] = None
    movimentacao: Optional[str] = None
    foro: Optional[str] = None
    vara: Optional[str] = None
    especializacao: Optional[str] = None
    cargo: Optional[str] = None

    @field_validator('*')
    def empty_str_to_none(cls, v):
        """
        Se o valor for '', converte para None
        https://github.com/pydantic/pydantic/discussions/2687
        """
        if v == '':
            return None
        return v

    @field_validator('disponibilizacao')
    def convert_to_date(cls, v):
        """
        Converte para datetime, isoformat
        """
        if v == '':
            return None
        return datetime.strptime(v, '%d/%m/%Y').date().isoformat()


class RecebimentoIntimacoes2Grau(BaseModel):
    """
    Objeto "Intimações" de 2º Grau ou Turmas Recursais.

    Muito similar às Intimações de 1º Grau,
    sendo que a única diferença é que existe
    "seção" e "órgão julgador" ao invés de "foro" e "vara".
    """

    disponibilizacao: Optional[str] = date
    prazo_processual: Optional[int] = None
    numero_processo: Optional[str] = None
    classe_assunto_principal: Optional[str] = None
    movimentacao: Optional[str] = None
    secao: Optional[str] = None
    orgao_julgador: Optional[str] = None
    especializacao: Optional[str] = None
    cargo: Optional[str] = None

    @field_validator('*')
    def empty_str_to_none(cls, v):
        """
        Se o valor for '', converte para None
        https://github.com/pydantic/pydantic/discussions/2687
        """
        if v == '':
            return None
        return v

    @field_validator('disponibilizacao')
    def convert_to_date(cls, v):
        """
        Converte para datetime, isoformat
        """
        if v == '':
            return None
        return datetime.strptime(v, '%d/%m/%Y').date().isoformat()


class ConsultaIntimacoes2Grau(BaseModel):
    """
    Objeto "Intimações" de 2º Grau ou Turmas Recursais.

    Muito similar às Intimações de 1º Grau,
    sendo que a única diferença é que existe
    "seção" e "órgão julgador" ao invés de "foro" e "vara".

    Utilizado nas Consultas
    """

    disponibilizacao: Optional[str] = date
    data_intimacao: Optional[str] = date
    prazo_processual: Optional[int] = None
    numero_processo: Optional[str] = None
    classe_assunto_principal: Optional[str] = None
    recebido_por: Optional[str] = None
    movimentacao: Optional[str] = None
    # secao: Optional[str] = None
    # orgao_julgador: Optional[str] = None
    especializacao: Optional[str] = None
    cargo: Optional[str] = None

    @field_validator('*')
    def empty_str_to_none(cls, v):
        """
        Se o valor for '', converte para None
        https://github.com/pydantic/pydantic/discussions/2687
        """
        if v == '':
            return None
        return v

    @field_validator('disponibilizacao', 'data_intimacao')
    def convert_to_date(cls, v):
        """
        Converte para datetime, isoformat
        """
        if v == '':
            return None
        
        # É possível que, no menu Consultas, tenham processos sem registro da data de intimação
        # Nessa situação, vem "None"
        elif v is None:
            return None
        
        return datetime.strptime(v, '%d/%m/%Y').date().isoformat()
        # return datetime.strptime(v, '%d/%m/%Y').date().norma


if __name__ == '__main__':
    # intim = RecebimentoIntimacoes2Grau(
    #     disponibilizacao='24/09/2024',
    #     prazo_processual='2° Grau',
    #     numero_processo='Criminal',
    #     classe_assunto_principal='False',
    #     movimentacao='False',
    #     especializacao='False',
    #     cargo='Ambas',
    # )
    # print(intim)
    # print(intim.disponibilizacao, type(intim.disponibilizacao))

    intim = ConsultaIntimacoes2Grau(
        disponibilizacao='24/09/2024',
        # data_intimacao='24/11/2024',
        data_intimacao=None,
        prazo_processual=10,
        numero_processo='2336412-80.2024.8.26.0000',
        classe_assunto_principal='Revisão Criminal / Roubo Majorado',
        movimentacao='Parecer - Prazo - 10 Dias',
        especializacao='Criminal',
        cargo='020º PROCURADOR DE JUSTIÇA',
    )
    print(intim)
    print(intim.disponibilizacao, type(intim.disponibilizacao))



"""
Módulo com os componentes gerais,
que são usados em vários filtros do eSAJ
"""

import time
from typing import List

from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.by import By

from pyesaj.scraper.pom import PageElement


# flake8: noqa:303
# flake8: noqa:E501


class InputModelSearchSet(PageElement):
    """
    Representa os campos de *input*, onde é possível pesquisar
    as opções por meio de um *pop-up*.
    """

    # xPaths Fixos
    btn_fechar = (By.XPATH, '//input[@id="pbFechar"]')
    btn_selecionar = (By.XPATH, '//input[@id="pbSelecionar"]')
    btn_procurar = (By.XPATH, '//input[@id="pbProcurar"]')
    iframe = (By.XPATH, '//iframe[@id="layerFormConsulta"]')

    def __init__(self, driver):
        super().__init__(driver)
        # xpaths variáveis
        self.abre_consulta = (
            By.XPATH,
            '//td[@id="labelForo"]/following-sibling::td//tr//img[@title="Abre a consulta"]',
        )
        self.linhas_tabela = (
            By.XPATH,
            '//table[@id="tabelaResultado"]//td[@property="nmForo"]',
        )
        self.input_opcao = (By.XPATH, '//input[@id="entity.nmForo"]')
        self.lista_opcoes = None

    def get_options(self) -> List[str]:
        """
        Obtém opções de preenchimento.
        """

        try:
            self.click(self.abre_consulta)
            self.switch(self.iframe)
            self.lista_opcoes = [
                x.text for x in self.find_all(self.linhas_tabela)
            ]
            self.click(self.btn_fechar)
            return self.lista_opcoes

        except Exception as e:
            self.click(self.btn_fechar)
            raise ValueError('Erro.') from e

        finally:
            self.switch_back()

    def set_option(self, option) -> None:
        """
        Define a opção para preenchimento do campo.
        """

        if self.lista_opcoes is None:
            self.lista_opcoes = self.get_options()

        if option not in self.lista_opcoes:
            raise ValueError(f'Precisa ser uma opção {self.lista_opcoes}')

        try:
            self.click(self.abre_consulta)
            self.switch(self.iframe)
            # self.lista_foros = [x.text for x in self.find_all(self.linhas_tabela)]
            # Insere o Foro no campo de busca
            self.set(self.input_opcao, option)

            # Procura
            self.click(self.btn_procurar)

            # Se a linha for a que eu procuro, Clica!
            # TODO: Aperfeiçoar com BS4 (18.11.2024)
            for row in self.find_all(self.linhas_tabela):
                if row.text == option:
                    row.click()

            # Seleciona
            self.click(self.btn_selecionar)

        except Exception as e:
            raise ValueError(
                f'Precisa ser uma opção {self.lista_opcoes}'
            ) from e

        finally:
            self.switch_back()


class InputModelTree(PageElement):
    """
    Representa os campos de *input*, onde é possível pesquisar
    as opções por meio de um *pop-up* hierárquico.

    Usado apenas na pesquisa de "Classe" e/ou "Assunto Principal".
    """

    def __init__(self, driver):
        super().__init__(driver)
        # Variável
        self.abre_consulta = (
            By.XPATH,
            '//td[@id=""]/following-sibling::td//tr//img[@id="botaoProcurar_classes" and @title="Abre a consulta"]',
        )
        self.linhas_tabela = (
            By.XPATH,
            '//ul[@id="classes_tree"]//span[@class="node selectable checkable Unchecked"]',
        )
        self.input_opcao = (
            By.XPATH,
            '//input[@id="classes_treeSelectFilter"]',
        )
        self.btn_selecionar = (
            By.XPATH,
            '//div[@id="assuntos_treeSelectContainer"]//input[@class="spwBotaoDefaultGrid" and @value="Selecionar"]',
        )
        self.btn_procurar = (
            By.XPATH,
            '//div[@id="assuntos_treeSelectContainer"]//input[@id="filtroButton"]',
        )
        self.lista_opcoes = None

        # Default
        self.btn_fechar = (
            By.XPATH,
            '//div[@class="blockUI blockMsg blockPage"]//button[@id="popupModalBotaoFechar"]',
        )
        self.msg_loading = (By.XPATH, '//div[@id="popupModalCarregando"]')

    def get_options(self) -> List:
        """
        Obtém opções de preenchimento.
        """

        self.zoom(zoom=100)

        try:
            # Abre a Consulta
            self.click(locator=self.abre_consulta)

            # Aguarda Carregar
            loading = self.find_all(locator=self.msg_loading, wait=0)
            tries = 0
            while len(loading) > 0 and tries <= 12:
                time.sleep(5)
                loading = self.find_all(self.msg_loading, wait=0)
                tries += 1
                print(f'Tentativa #{tries}. Já aguardei 5 segundos')

            # Pega opções para checar
            rows = self.find_all(self.linhas_tabela, wait=0)
            # print(len(rows))
            self.lista_opcoes = [x.text for x in rows]
            return self.lista_opcoes

        except Exception as e:
            raise ValueError('Problemas para obter os itens.') from e
            # CheckError(self).has_errors()

        finally:
            # Fecha a Consulta
            self.click(locator=self.btn_fechar)

    def set_option(self, option) -> None:
        """
        Define a opção para preenchimento do campo.
        """
        self.zoom(zoom=100)
        if self.lista_opcoes is None:
            self.lista_opcoes = self.get_options()

        if option not in self.lista_opcoes:
            raise ValueError(f'Precisa ser uma opção {self.lista_opcoes}')

        try:
            self.click(self.abre_consulta)

            # Aguarda Carregar
            loading = self.find_all(locator=self.msg_loading, wait=0)
            tries = 0
            while len(loading) > 0 and tries <= 12:
                time.sleep(5)
                loading = self.find_all(self.msg_loading, wait=0)
                tries += 1
                print(f'Tentativa #{tries}. Já aguardei 5 segundos')

            # Define o termo a ser pesquisado
            self.set(self.input_opcao, option)

            # Clica em Procurar
            self.click(self.btn_procurar)
            time.sleep(10)

            # Se a linha for a que eu procuro, Clica!
            for row in self.find_all(self.linhas_tabela, wait=0):
                if row.text == option:
                    row.click()
                    break

        except Exception as e:
            raise ValueError('Erro ao definir opção.') from e

        finally:
            # Preciso fazer essa gambiarra pois o eSAJ é uma merda
            # e esconde o botão
            # Em headless deve funcionar sem essa necessidade, mas como
            # não estamos em headless, foi necessário!
            self.zoom(zoom=50)
            time.sleep(2)
            # self.set(locator=self.input_opcao, value=Keys.RETURN)
            # self.click(locator=self.btn_selecionar)
            ActionChains(self.driver).move_to_element(
                self.find(locator=self.btn_selecionar)
            ).click().perform()
            self.zoom(zoom=100)


if __name__ == '__main__':
    pass



"""
Módulo com a página de
"Consulta de Intimações Recebidas"
"""

from datetime import date, datetime
from typing import List, Literal, Union

from selenium.webdriver.common.by import By

from pyesaj.scraper.pom import Page, PageElement


# from selenium.webdriver.common.keys import Keys
# from selenium.webdriver.common.action_chains import ActionChains
# import time
# from typing import Literal
# from selenium.webdriver.common.action_chains import ActionChains
# from selenium.webdriver.common.by import By


class Consulta(Page):
    """
    Página "Intimações On-line > Consulta de Intimações Recebidas"
    """

    url = 'https://esaj.tjsp.jus.br/intimacoesweb/abrirConsultaAtosRecebidos.do'

    def __init__(self, driver):
        super().__init__(driver)
        self.go_to(self.url)


class Periodo(PageElement):
    """
    Representa o menu *dropdown* "Em nome de", que define
    qual o perfil do usuário que será utilizado.

    Pode ser pesquisa de intimações recebidas pelo nome do usuário,
    pessoa física, ou a pesquisa pelo
    "Ministério Público do Estado de São Paulo",
    caso o usuário seja funcionário dessa instituição.
    """

    dt_inicio = (By.XPATH, '//input[@name="dadosConsulta.dtInicioPeriodo"]')
    dt_fim = (By.XPATH, '//input[@name="dadosConsulta.dtFimPeriodo"]')

    def de(self, data: Union[date, str]):
        """
        Define a Data de Início do período de consulta.
        Precisa estar em formato *date* ou *str*, no padrão "%d/%m/%Y".
        """
        if isinstance(data, str):
            data = datetime.strptime(data, '%d/%m/%Y').date()

        elif isinstance(data, date):
            pass

        else:
            raise Exception(
                'Precisa ser formato date ou str no padrão "%d/%m/%Y"'
            )

        self.set(self.dt_inicio, data.strftime(format='%d/%m/%Y'))

    def ate(self, data: Union[date, str]):
        """
        Define a Data Fim do período de consulta.
        Precisa estar em formato *date* ou *str*, no padrão "%d/%m/%Y".
        """
        if isinstance(data, str):
            data = datetime.strptime(data, '%d/%m/%Y').date()

        elif isinstance(data, date):
            pass

        else:
            raise Exception(
                'Precisa ser formato date ou str no padrão "%d/%m/%Y"'
            )

        # data_inicio = self.get_text(locator=self.dt_inicio)

        # print('Data Início ', data_inicio)
        self.set(self.dt_fim, data.strftime(format='%d/%m/%Y'))

    def define_intervalo(self, de, ate):
        """
        Define a Data de Início e Fim do período de consulta.
        As datas precisam estar em formato *date* ou *str*, no padrão "%d/%m/%Y".
        """

        if isinstance(de, str):
            de = datetime.strptime(de, '%d/%m/%Y').date()

        elif isinstance(de, date):
            pass

        else:
            raise Exception(
                'Precisa ser formato date ou str no padrão "%d/%m/%Y"'
            )

        if isinstance(ate, str):
            ate = datetime.strptime(ate, '%d/%m/%Y').date()

        elif isinstance(ate, date):
            pass

        else:
            raise Exception(
                'Precisa ser formato date ou str no padrão "%d/%m/%Y"'
            )

        # Avalia se o
        if de <= ate:
            pass

        elif de > ate:
            raise Exception(
                'Data de início precisa ser menor ou igual que data fim!'
            )

        # Define
        self.de(data=de)
        self.ate(data=ate)


class CienciaAto(PageElement):
    """
    Representa o menu *dropdown* "Ciência do Ato", que define
    qual o tip da ciência do ato desejada.
    """

    selector_em_nome_de = (
        By.XPATH,
        '//select[@name="dadosConsulta.formaCienciaIntimacao"]',
    )

    def get_options(self) -> List[str]:
        """
        Obtem opções de preenchimento.
        """
        return self.create_select_options(self.selector_em_nome_de)

    def set_option(self, option) -> None:
        """
        Define a opção para preenchimento do campo.
        """
        list_options = self.get_options()
        if option not in list_options:
            raise ValueError(f'Precisa ser uma opção {list_options}')

        self.select(self.selector_em_nome_de, option)


class Situacao(PageElement):
    """
    Representa o *radio button* "Situação".
    """

    cumprida = (
        By.XPATH,
        '//input[@name="entity.ato.flCumprido" and @value="S"]',
    )
    pendente = (
        By.XPATH,
        '//input[@name="entity.ato.flCumprido" and @value="N"]',
    )
    ambas = (By.XPATH, '//input[@name="entity.ato.flCumprido" and @value]')

    def set_option(
        self, situacao: Literal['Cumprida', 'Pendente', 'Ambas']
    ) -> None:
        """
        Define a opção para preenchimento do campo.
        """

        if situacao == 'Cumprida':
            self.click(locator=self.cumprida)

        elif situacao == 'Pendente':
            self.click(locator=self.pendente)

        elif situacao == 'Ambas':
            all = self.find_all(locator=self.ambas)
            all[2].click()


if __name__ == '__main__':
    import os
    import time

    from dotenv import load_dotenv

    import pyesaj as esaj

    # Credenciais
    load_dotenv()
    USERNAME = os.getenv('USERNAME_TJSP')
    PASSWORD = os.getenv('PASSWORD_TJSP')

    # Cria Driver
    driver2 = esaj.scraper.webdriver.Firefox(verify_ssl=False)

    # Entra no eSAJ e loga
    log = esaj.scraper.page.Login(driver=driver2)
    log.login(username=USERNAME, password=PASSWORD)

    # Consulta Intimações
    esaj.scraper.page.intim.Consulta(driver=driver2)

    # em_nome = esaj.scraper.page.intim.consulta.EmNomeDe(driver=driver)
    # print(em_nome.get_options())

    time.sleep(5)
    driver2.quit()



"""
Módulo com a página de login
"""

import time
from urllib.parse import urlparse

from selenium.webdriver.common.by import By

from pyesaj.scraper.pom import Page, PageElement


class Username(PageElement):
    """
    Representa o campo de *input* "CPF/CNPJ",
    necessário para acessar o e-SAJ.
    """

    input_username = (By.XPATH, '//input[@id="usernameForm"]')
    nome_usuario = (By.XPATH, '//span[@id="identificacao"]/strong')

    def set_username(self, username: str) -> None:
        """
        Define o "usuário" para logar no e-SAJ.
        """
        self.set(self.input_username, username)

    def get_username(self) -> str:
        """
        Obtem o "usuário" logado no e-SAJ.
        """
        return self.get_text(self.nome_usuario)


class Password(PageElement):
    """
    Representa o campo de *input* "Senha", necessário para acessar
    o e-SAJ.
    """

    input_password = (By.XPATH, '//input[@id="passwordForm"]')

    def set_password(self, password) -> None:
        """
        Define a "senha" para logar no e-SAJ.
        """
        self.set(self.input_password, password)


class Token(PageElement):
    """
    Representa o campo de *input* "Senha", necessário para acessar
    o e-SAJ.
    """

    input_token = (By.XPATH, '//input[@id="tokenInformado"]')

    def set_token(self, token) -> None:
        """
        Define a "senha" para logar no e-SAJ.
        """
        self.set(self.input_token, token)


class Login(Page):
    """
    Página "eSAJ > Login"
    https://esaj.tjsp.jus.br/sajcas/login
    """

    url = 'https://esaj.tjsp.jus.br/esaj/portal.do?servico=740000'
    identificar_se = (By.XPATH, "//a[text()='Identificar-se']")
    btn_entrar = (By.XPATH, '//input[@id="pbEntrar"]')
    btn_enviar_token = (By.XPATH, '//button[@id="btnEnviarToken"]')

    msg_login = (
        By.XPATH,
        '//table[@class="tabelaMensagem"]//td[@id="mensagemRetorno"]',
    )
    msg_token = (
        By.XPATH,
        '//div[@id="avisoTokenInvalido"]',
    )

    def __init__(self, driver) -> None:
        super().__init__(driver)
        self.go_to(url=self.url)
        self.user = None

    def login_1_etapa(self, username, password) -> None:
        user = Username(self.driver)
        passwd = Password(self.driver)

        if user.get_username() == 'Identificar-se':
            # Clica em Identificar-se
            self.click(self.identificar_se)

            # Insere Parâmetros
            user.set_username(username=username)
            passwd.set_password(password=password)

            # Faz o Login
            self.click(self.btn_entrar)

        else:
            # Define Variável
            self.user = user.get_username()

            # Mensagem
            print(f'Olá "{self.user}", você já estava logado no e-SAJ!')

    def login_2_etapa(self, token: int | str) -> None:
        user = Username(self.driver)
        token_obj = Token(self.driver)

        if user.get_username() == 'Identificar-se':
            # Define o Token
            token_obj.set_token(token=str(token))

            # Clica em "Enviar" token
            self.click(self.btn_enviar_token)

            # Tentativa de trabalhar com a inserção de um token errado
            # try:
            #     msg = self.get_text(locator=self.msg_token, wait=3)
            # except:
            #     msg = ''
            #
            # if msg == 'O código informado está inválido. Verifique sua caixa de e-mail para conferir o código que foi enviado ou tente reenviar o código novamente.':
            #     raise Exception(msg)

            # Define Variável
            self.user = user.get_username()

            # Pega URL
            path = urlparse(url=self.driver.current_url).path

            # Se logou!
            if path == '/esaj/portal.do':
                print(f'Olá "{self.user}", você está logado no e-SAJ!')

            # Se não logou!
            elif path == '/sajcas/login':
                msg = self.get_text(locator=self.msg_login)
                if msg == 'Usuário e/ou senha inválidos':
                    raise Exception(msg)

        else:
            # Define Variável
            self.user = user.get_username()

            # Mensagem
            print(f'Olá "{self.user}", você já estava logado no e-SAJ!')

    # def login(self, username, password) -> None:
    #     """
    #     Faz o login
    #     Usado até 24/03/2025, quando não havia login em duas etapas
    #     """
    #
    #     user = Username(self.driver)
    #     passwd = Password(self.driver)
    #
    #     if user.get_username() == 'Identificar-se':
    #         # Clica em Identificar-se
    #         self.click(self.identificar_se)
    #
    #         # Insere Parâmetros
    #         user.set_username(username=username)
    #         passwd.set_password(password=password)
    #
    #         # Faz o Login
    #         self.click(self.btn_entrar)
    #
    #         # Define Variável
    #         self.user = user.get_username()
    #
    #         # Pega URL
    #         path = urlparse(url=self.driver.current_url).path
    #
    #         # Se logou!
    #         if path == '/esaj/portal.do':
    #             print(f'Olá "{self.user}", você está logado no e-SAJ!')
    #
    #         # Se não logou!
    #         elif path == '/sajcas/login':
    #             msg = self.get_text(locator=self.msg_login)
    #             if msg == 'Usuário e/ou senha inválidos':
    #                 raise Exception(msg)
    #
    #     else:
    #         # Define Variável
    #         self.user = user.get_username()
    #
    #         # Mensagem
    #         print(f'Olá "{self.user}", você já estava logado no e-SAJ!')


if __name__ == '__main__':
    import os
    import time

    from dotenv import load_dotenv

    import pyesaj as esaj

    # Credenciais
    load_dotenv()
    USERNAME = os.getenv('USERNAME_TJSP')
    PASSWORD = os.getenv('PASSWORD_TJSP')

    # Cria Driver
    driver2 = esaj.scraper.webdriver.Firefox(
        # driver_path=paths.driver_path,
        # logs_path=paths.app_logs_path,
        # down_path=paths.driver_path,
        # headless=False,
        verify_ssl=False
    )

    # Faz Login
    log = esaj.scraper.page.Login(driver=driver2)

    # Não é mais usado
    # log.login(username=USERNAME, password=PASSWORD)

    # Etapa 1
    log.login_1_etapa(username=USERNAME, password=PASSWORD)

    # Etapa 2
    log.login_2_etapa(token=365745)

    # Username(driver).set_username(username=USERNAME)

    time.sleep(1)
    # print('Agora eu dou refresh')
    # driver2.refresh()

    print('Agora eu fecho')
    driver2.quit()



"""

"""

from .scraper import params
# from .scraper import page, webdriver, search, outros

#
#
# from .page.cadastro import DadosCadastrais, GerenciamentoPerfis
# from .page.consulta_processos import (
#     CDA,
#     OAB,
#     ConsultaPor,
#     ConsultaProcesso,
#     Consultar,
#     DocumentoParte,
#     NomeAdvogado,
#     NomeParte,
#     NumCartaPrecatoria,
#     NumDocDelegacia,
#     NumeroUnificado,
# )
# from .page.login import Login
# from .page.recebe_initimacoes import (
#     Acoes,
#     Area,
#     AssuntoPrincipal,
#     Cargo,
#     CheckError,
#     Classe,
#     ConsultarIntimacoes,
#     EmNomeDe,
#     Especializacao,
#     Foro,
#     Instancia,
#     Intimacoes,
#     NaturezaComunicacao,
#     OrgaoJulgador,
#     Paginacao,
#     Processo,
#     Secao,
#     Tabela,
#     TipoParticipacao,
#     Vara,
# )



"""
Módulo com a página de cadastro do eSAJ
Para pegar o que o usuário tem autorização para fazer
"""

import re
import time

from selenium.webdriver.common.by import By

from pyesaj.scraper.pom import Page, PageElement


# from selenium.webdriver.common.keys import Keys
# from selenium.webdriver.common.action_chains import ActionChains


# flake8: noqa:303
# flake8: noqa:E501


class Recebimento(Page):
    """
    Página "Intimações On-line > Recebimento de Intimações Eletrônicas"
    """

    url = 'https://esaj.tjsp.jus.br/intimacoesweb/abrirConsultaAtosNaoRecebidos.do'

    def __init__(self, driver):
        super().__init__(driver)
        self.go_to(self.url)


class Paginacao(PageElement):
    # Paginação Botões
    btn_primeira_pagina = (
        By.XPATH,
        '//div[@id="rodapeGridPaginada"]//span[@id="botoesRodapeGrid"]//input[@title="Primeira página"]',
    )
    btn_pagina_anterior = (
        By.XPATH,
        '//div[@id="rodapeGridPaginada"]//span[@id="botoesRodapeGrid"]//input[@title="Página anterior"]',
    )
    btn_proxima_pagina = (
        By.XPATH,
        '//div[@id="rodapeGridPaginada"]//span[@id="botoesRodapeGrid"]//input[@title="Próxima página"]',
    )
    btn_ultima_pagina = (
        By.XPATH,
        '//div[@id="rodapeGridPaginada"]//span[@id="botoesRodapeGrid"]//input[@title="Última página"]',
    )
    # Páginas
    pagina_atual_xpath = (
        By.XPATH,
        '//div[@id="rodapeGridPaginada"]//span[@id="botoesRodapeGrid"]//input[@id="gridAtosNaoRecebidosUsuario_display"] | //div[@id="rodapeGridPaginada"]//span[@id="botoesRodapeGrid"]//input[@id="gridAtosUsuario_display"]',
    )
    total_paginas_xpath = (
        By.XPATH,
        '//div[@id="rodapeGridPaginada"]//span[@id="botoesRodapeGrid"]//c[@id="gridAtosNaoRecebidosUsuario_nuTotalPags"] | //div[@id="rodapeGridPaginada"]//span[@id="botoesRodapeGrid"]//c[@id="gridAtosUsuario_nuTotalPags"]',
    )
    # Intimações
    n_intimacoes_inferior = (
        By.XPATH,
        '//div[@id="rodapeGridPaginada"]//b[@id="gridAtosNaoRecebidosUsuarioLimInferior"] | //div[@id="rodapeGridPaginada"]//b[@id="gridAtosUsuarioLimInferior"]',
    )
    n_intimacoes_superior = (
        By.XPATH,
        '//div[@id="rodapeGridPaginada"]//b[@id="gridAtosNaoRecebidosUsuarioLimSuperior"] | //div[@id="rodapeGridPaginada"]//b[@id="gridAtosUsuarioLimSuperior"]',
    )
    n_intimacoes_total = (By.XPATH, '//div[@id="rodapeGridPaginada"]')

    def infos(self):
        """
        Pega Página atual
        pega total de páginas
        Vai pra próxima, desde que seja menor que o número total de paginas
        Aguarda carregar
        """
        # Número de Páginas
        num_page_atual = self.attribute(
            locator=self.pagina_atual_xpath, attribute='value'
        )
        num_page_atual = int(num_page_atual)
        num_page_total_atual = self.get_text(self.total_paginas_xpath)
        num_page_total_atual = int(num_page_total_atual)
        # print(f'Páginas: de {num_page_atual} à {num_page_total_atual}')

        # Número de Intimações
        n_inf = self.get_text(locator=self.n_intimacoes_inferior)
        n_sup = self.get_text(locator=self.n_intimacoes_superior)

        # Converte em inteiros
        n_inf = int(n_inf)
        n_sup = int(n_sup)

        # Paginação
        paginacao = self.get_text(locator=self.n_intimacoes_total)
        paginacao = paginacao.split('\n')[0]

        list_results = re.findall(r'de (\w+)', paginacao)
        if len(list_results) == 1:
            n_intimacoes = int(list_results[0])

        else:
            raise ValueError(
                'Problema na obtenção do número total de intimações.'
            )
        # print(f'Intimações: de {n_inf} à {n_sup} ({n_intimacoes})')
        return {
            'num_intimacoes_de': n_inf,
            'num_intimacoes_até': n_sup,
            'num_intimacoes_total': n_intimacoes,
            'num_pagina_atual': num_page_atual,
            'num_pagina_total': num_page_total_atual,
        }

    def proxima_pagina(self) -> int:
        """
        Pega Página atual
        pega total de páginas
        Vai pra próxima, desde que seja menor que o númeor total de paginas
        Aguarda carregar
        """
        pages_info = self.infos()
        n_pagina_inicio = pages_info['num_pagina_atual']
        n_pagina_atual = pages_info['num_pagina_atual']
        n_pagina_quero = pages_info['num_pagina_atual'] + 1
        n_pagina_total = pages_info['num_pagina_total']

        if n_pagina_atual == n_pagina_total:
            print('Já estou na última página')
            return n_pagina_atual

        if (
            n_pagina_inicio < n_pagina_total
            and n_pagina_inicio < n_pagina_quero
        ):

            while n_pagina_atual != n_pagina_quero:
                # Avança
                self.click(locator=self.btn_proxima_pagina)
                time.sleep(2)

                # Pega Página
                n_pagina_atual = self.infos()['num_pagina_atual']
                n_pagina_changed = self.infos()['num_pagina_atual']
                print(
                    f'Estava na página {n_pagina_inicio}. Vim para {n_pagina_changed}'
                )
            return n_pagina_atual

        if n_pagina_atual > n_pagina_total:
            raise RuntimeError('Como estou em uma página que não existe?')

    def volta_pagina(self) -> int:
        """
        Pega Página atual
        pega total de páginas
        Vai pra próxima, desde que seja menor que o número total de paginas
        Aguarda carregar
        """
        pages_info = self.infos()
        n_pagina_inicio = pages_info['num_pagina_atual']
        n_pagina_atual = pages_info['num_pagina_atual']
        n_pagina_quero = pages_info['num_pagina_atual'] - 1
        n_pagina_total = pages_info['num_pagina_total']

        if n_pagina_inicio == 1:
            print('Já estou na primeira página')
            return n_pagina_atual

        if (
            # n_pagina_inicio <= n_pagina_total
            # and n_pagina_inicio > n_pagina_quero
            n_pagina_total
            >= n_pagina_inicio
            > n_pagina_quero
        ):

            while n_pagina_atual != n_pagina_quero:
                # Retrocede
                self.click(locator=self.btn_pagina_anterior)
                time.sleep(2)

                # Pega Página
                n_pagina_atual = self.infos()['num_pagina_atual']
                n_pagina_changed = self.infos()['num_pagina_atual']
                print(
                    f'Estava na página {n_pagina_inicio}. Vim para {n_pagina_changed}'
                )
            return n_pagina_atual

        if n_pagina_atual > n_pagina_total:
            raise RuntimeError('Como estou em uma página que não existe?')

    def vai_primeira(self) -> int:
        """
        Vai para a primeira página.
        """
        pages_info = self.infos()
        # n_pagina_inicio = pages_info['num_pagina_atual']
        n_pagina_atual = pages_info['num_pagina_atual']
        n_pagina_quero = 1
        # n_pagina_total = pages_info['num_pagina_total']

        while n_pagina_atual != n_pagina_quero:
            self.click(locator=self.btn_primeira_pagina)
            time.sleep(2)

            # Pega Página
            n_pagina_atual = self.infos()['num_pagina_atual']
        return n_pagina_atual

    def vai_ultima(self):
        """
        Vai para a última página.
        """
        pages_info = self.infos()
        # n_pagina_inicio = pages_info['num_pagina_atual']
        n_pagina_atual = pages_info['num_pagina_atual']
        # n_pagina_quero = 1
        n_pagina_quero = pages_info['num_pagina_total']

        while n_pagina_atual != n_pagina_quero:
            self.click(locator=self.btn_ultima_pagina)
            time.sleep(2)

            # Pega Página
            n_pagina_atual = self.infos()['num_pagina_atual']
        return n_pagina_atual


if __name__ == '__main__':
    import os

    from dotenv import load_dotenv

    import pyesaj as esaj

    # Credenciais
    load_dotenv()
    USERNAME = os.getenv('USERNAME_TJSP')
    PASSWORD = os.getenv('PASSWORD_TJSP')

    # Cria Driver
    driver = esaj.webdriver.Firefox(verify_ssl=False)

    # Entra no eSAJ e loga
    log = esaj.page.Login(driver=driver)
    log.login(username=USERNAME, password=PASSWORD)

    # Intimações
    intim = esaj.page.intim.Recebimento(driver=driver)

    intim_em_nome = esaj.page.intim.EmNomeDe(driver=driver)
    intim_em_nome.set_option(option='Ministério Público do Estado de São Paulo')

    # Instância: 1º Grau
    intim_inst = esaj.page.intim.Instancia(driver=driver)
    intim_inst.set_option(instancia='Primeiro Grau')

    # # Foro
    # intim_foro = esaj.page.intim.recebe.Foro(driver=driver)
    # print(intim_foro.get_options())
    # intim_foro.set_option(option='Foro de Campinas')
    #
    # # Vara
    # intim_vara = esaj.page.intim.recebe.Vara(driver=driver)
    # print(intim_vara.get_options())
    # intim_vara.set_option(option='2ª Vara de Família e Sucessões')
    #
    # # Especialização
    # intim_esp = esaj.page.intim.recebe.Especializacao(driver=driver)
    # print(intim_esp.get_options())
    # intim_esp.set_option(option='Registros Públicos')

    # Instância: 2º Grau
    intim_inst = esaj.page.intim.Instancia(driver=driver)
    intim_inst.set_option(instancia='Segundo Grau')

    # Seção
    intim_secao = esaj.page.intim.Secao(driver=driver)
    print(intim_secao.get_options())
    intim_secao.set_option(option='Direito Privado 2')

    # Órgão Julgador
    # intim_org = esaj.page.intim.recebe.OrgaoJulgador(driver=driver)
    # print(intim_org.get_options())

    # Consultar
    consulta = pyesaj.scraper.page.intim.recebe.ConsultarIntimacoes(driver=driver)
    consulta.consultar()
    time.sleep(5)

    # Paginação
    pag = pyesaj.scraper.page.intim.recebe.Paginacao(driver=driver)
    print(pag.infos())

    time.sleep(5)
    driver.quit()
